"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseSourceFile = parseSourceFile;
const ts = require("typescript");
function parseSourceFile(file) {
    const result = {
        imports: [],
        classes: []
    };
    walker(file);
    return result;
    function walker(node) {
        switch (node.kind) {
            case ts.SyntaxKind.ImportDeclaration:
                importsWalker(node);
                break;
            case ts.SyntaxKind.ClassDeclaration:
                classWalker(node);
                break;
            default:
                ts.forEachChild(node, walker);
        }
    }
    function classWalker(node) {
        const klass = {
            name: node.name && node.name.escapedText,
            dependencies: []
        };
        ts.forEachChild(node, (child) => {
            // deps via constructor
            if (child.kind === ts.SyntaxKind.Constructor) {
                const constructor = child;
                constructor.parameters.forEach(param => {
                    klass.dependencies.push({
                        name: param.name.getText(),
                        type: param.type && param.type.getText(),
                        token: extractInjectionToken(param)
                    });
                });
            }
            // deps from properties
            if (child.kind === ts.SyntaxKind.PropertyDeclaration) {
                const prop = child;
                prop.forEachChild(propChild => {
                    var _a, _b, _c, _d, _e;
                    if (propChild.kind === ts.SyntaxKind.CallExpression) {
                        const text = propChild.getText();
                        if (text.startsWith('inject')) {
                            let token;
                            let type = prop.type && prop.type.getText();
                            if (text.startsWith('inject<')) {
                                token = (_b = (_a = propChild.getChildAt(5)) === null || _a === void 0 ? void 0 : _a.getChildAt(0)) === null || _b === void 0 ? void 0 : _b.getText().trim();
                                if (!type) {
                                    type = (_c = propChild.getChildAt(2)) === null || _c === void 0 ? void 0 : _c.getText();
                                }
                            }
                            else {
                                token = (_e = (_d = propChild.getChildAt(2)) === null || _d === void 0 ? void 0 : _d.getChildAt(0)) === null || _e === void 0 ? void 0 : _e.getText().trim();
                            }
                            // support string tokens
                            if (token.startsWith('\'') || token.startsWith('"')) {
                                const typeFromToken = !type && token.match(/ProviderToken<(.+)>/);
                                if (typeFromToken && typeFromToken[1]) {
                                    type = typeFromToken[1];
                                }
                                // only use the string part for DI
                                token = token.substr(0, token.indexOf(token[0], 1) + 1);
                            }
                            else if (!type) {
                                type = token;
                            }
                            // special case of injection tokens following upper case convention
                            if (!prop.type && type && /^[A-Z]+$/.test(type.replace(/_/g, ''))) {
                                type = `typeof ${type} extends ProviderToken<infer T> ? T : unknown`;
                                if (!result.imports.find(i => i.names.includes('ProviderToken'))) {
                                    result.imports.push({
                                        path: '@angular/core',
                                        names: ['ProviderToken'],
                                    });
                                }
                            }
                            klass.dependencies.push({
                                name: prop.name.getText(),
                                type,
                                token,
                            });
                        }
                    }
                });
            }
        });
        result.classes.push(klass);
    }
    function extractInjectionToken(param) {
        let token;
        // older TS version
        if (param.decorators) {
            param.decorators.forEach(decoratorWalker);
        }
        else if (param.modifiers) {
            param.modifiers.forEach(decoratorWalker);
        }
        return token;
        function decoratorWalker(node) {
            switch (node.kind) {
                case ts.SyntaxKind.CallExpression:
                    const callExpression = node;
                    if (callExpression.expression.getText() === 'Inject' && callExpression.arguments.length > 0) {
                        token = callExpression.arguments[0].getText();
                    }
                    break;
                default:
                    ts.forEachChild(node, decoratorWalker);
            }
        }
    }
    function importsWalker(node) {
        const names = [];
        if (node.importClause) {
            ts.forEachChild(node.importClause, (child) => {
                ts.forEachChild(child, (element) => {
                    names.push(element.getText());
                });
            });
        }
        result.imports.push({
            path: node.moduleSpecifier.getText(),
            names
        });
    }
}
